@@ -1,9 +1,16 @@
-orderMCMCplus1<-function(n,startorder,iterations,stepsave,moveprobs,parenttable,scoretable,aliases,numparents,rowmaps,
-                         plus1lists,scoresmatrices,numberofparentsvec,gamma=1){
+#implements order MCMC on an extended search space sampling version
+orderMCMCplus1<-function(n,nsmall,startorder,iterations,stepsave,moveprobs,parenttable,scoretable,aliases,numparents,rowmaps,
+                         plus1lists,scoresmatrices,numberofparentsvec,gamma=1,bgnodes,bglogscore){
+  
+  if(nsmall<n) {
+    mainnodes<-c(1:n)[-bgnodes]
+  } else {mainnodes<-c(1:n)}
+  
   currentpermy<-startorder #starting order represented as a permutation
-  currentorderscores<-orderscorePlus1(n,startorder,c(1:n),parenttable,aliases,numparents,rowmaps,plus1lists,scoretable,scoresmatrices,currentpermy) #starting score
-  currenttotallogscore<-sum(currentorderscores$totscores) #log total score of all DAGs in the starting order
-  currentDAG<-samplescoreplus1(n,currentorderscores,plus1lists,scoretable,scoresmatrices,parenttable,numberofparentsvec,aliases) #score of a single DAG sampled from the starting order
+  currentorderscores<-orderscorePlus1(n,startorder[1:nsmall],c(1:nsmall),parenttable,aliases,numparents,rowmaps,plus1lists,scoretable,scoresmatrices,currentpermy) #starting score
+  currenttotallogscore<-sum(currentorderscores$totscores[mainnodes],bglogscore) #log total score of all DAGs in the starting order
+  currentDAG<-samplescoreplus1(n,mainnodes,currentorderscores,plus1lists,scoretable,
+                               scoresmatrices,parenttable,numberofparentsvec,aliases,bglogscore) #score of a single DAG sampled from the starting order
   
   L1 <- list() # stores the adjacency matrix of a DAG sampled from the orders
   L2 <- list() # stores its log BGe score of a DAG
@@ -33,16 +40,16 @@ orderMCMCplus1<-function(n,startorder,iterations,stepsave,moveprobs,parenttable,
         proposedpermy<-currentpermy #sample a new order by swapping two elements
         switch(as.character(chosenmove),
                "1"={ # swap any two elements at random
-                 sampledelements<-sample.int(n,2,replace=FALSE) #chosen at random
+                 sampledelements<-sample.int(nsmall,2,replace=FALSE) #chosen at random
                  
                },
                "2"={ # swap any adjacent elements
-                 k<-sample.int(n-1,1) #chose the smallest at random
+                 k<-sample.int(nsmall-1,1) #chose the smallest at random
                  sampledelements<-c(k,k+1)
                  
                },
                "3"={ # swap any adjacent elements
-                 sampledpos<-sample.int(n,1)
+                 sampledpos<-sample.int(nsmall,1)
                },
                {# if neither is chosen, we have a problem
                  print('The move sampling has failed!')
@@ -64,7 +71,7 @@ orderMCMCplus1<-function(n,startorder,iterations,stepsave,moveprobs,parenttable,
             currentorderscores$allowedlists[rescorenodes]<-proposedorderrescored$allowedlists[rescorenodes]
           }
         } else if (chosenmove==3) {
-          neworder<-positionscorePlus1(n,currentorderscores,sampledpos,currentpermy,aliases,
+          neworder<-positionscorePlus1(n,nsmall,currentorderscores,sampledpos,currentpermy,aliases,
                                        rowmaps,plus1lists,numparents,scoretable,scoresmatrices)
           currentpermy<-neworder$order
           currentorderscores<-neworder$score
@@ -72,7 +79,8 @@ orderMCMCplus1<-function(n,startorder,iterations,stepsave,moveprobs,parenttable,
         }
       }
     }
-    currentDAG<-samplescoreplus1(n,currentorderscores,plus1lists,scoretable,scoresmatrices,parenttable,numberofparentsvec,aliases)
+    currentDAG<-samplescoreplus1(n,mainnodes,currentorderscores,plus1lists,scoretable,scoresmatrices,
+                                 parenttable,numberofparentsvec,aliases,bglogscore)
     L1[[z]]<-currentDAG$incidence #store adjacency matrix of a sampled DAG each 'stepsave'
     L2[[z]]<-currentDAG$logscore #and log score of a sampled DAG
     L3[[z]]<-currenttotallogscore #and the current order score
